"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
/* eslint-disable @typescript-eslint/no-require-imports */
/* eslint-disable @typescript-eslint/no-explicit-any */
const trino_client_1 = require("trino-client");
const BaseClient = require("knex/lib/dialects/mysql/index.js");
const { formatQuery, } = require("knex/lib/execution/internal/query-executioner");
const QueryCompiler_MySQL = require("knex/lib/dialects/mysql/query/mysql-querycompiler.js");
// move offset to front of limit
QueryCompiler_MySQL.prototype.offset = function () {
    const noLimit = !this.single.limit && this.single.limit !== 0;
    const noOffset = !this.single.offset;
    if (noOffset)
        return "";
    let offset = `offset ${noOffset ? "0" : this._getValueOrParameterFromAttribute("offset")}`;
    if (!noLimit) {
        offset += ` limit ${this._getValueOrParameterFromAttribute("limit")}`;
    }
    return offset;
};
QueryCompiler_MySQL.prototype.limit = function () {
    const noLimit = !this.single.limit && this.single.limit !== 0;
    const noOffset = !this.single.offset;
    if (!noOffset || noLimit)
        return "";
    this.single.limit = noLimit ? -1 : this.single.limit;
    return `limit ${this._getValueOrParameterFromAttribute("limit")}`;
};
class ClientAtlasSqlOdbcImpl extends BaseClient {
    constructor(config) {
        super(Object.assign(Object.assign({}, config), { 
            // Enforce a single connection:
            pool: { min: 1, max: 1 }, connection: {} }));
    }
    _driver() {
        this.trino = trino_client_1.Trino.create(this.config.trino);
    }
    // Acquire a connection from the pool.
    acquireConnection() {
        return __awaiter(this, void 0, void 0, function* () {
            return this.trino;
        });
    }
    destroyRawConnection(connection) {
        return __awaiter(this, void 0, void 0, function* () {
            // There is only one connection, if this one goes shut down the database
            yield connection.close();
        });
    }
    // Releases a connection back to the connection pool,
    // returning a promise resolved when the connection is released.
    releaseConnection() { }
    // Destroy the current connection pool for the client.
    destroy(callback) {
        return __awaiter(this, void 0, void 0, function* () {
            callback();
        });
    }
    _query(connection, obj) {
        return __awaiter(this, void 0, void 0, function* () {
            var _a;
            if (!obj.sql)
                throw new Error("The query is empty");
            // let query = obj.sql;
            // obj.bindings.forEach((binding, index) => {
            //   let value;
            //   switch (typeof binding) {
            //     case "string":
            //       value = `'${binding}'`;
            //       break;
            //     case "boolean":
            //       value = binding ? "TRUE" : "FALSE";
            //       break;
            //     default:
            //       value = binding;
            //   }
            //   query = query.replace(`?`, value);
            // });
            const query = formatQuery(obj.sql, obj.bindings, "UTC", this).replaceAll("`", '"');
            const iter = yield connection.query(query);
            const result = yield iter.next();
            if (!((_a = result.value.data) === null || _a === void 0 ? void 0 : _a.length)) {
                obj.response = [];
                return obj;
            }
            const formatedResults = result.value.data.map((resultArr) => {
                return resultArr.reduce((memo, resultArrVal, index) => {
                    memo[result.value.columns[index].name] = resultArrVal;
                    return memo;
                }, {});
            });
            obj.response = formatedResults;
            return obj;
        });
    }
    // Process the response as returned from the query.
    processResponse(obj) {
        if (obj == null)
            return;
        const { response } = obj;
        const { method } = obj;
        switch (method) {
            case "select":
                return response;
            case "first":
                return response[0];
            case "del":
            case "update":
            case "counter":
                return response;
            default:
                return response;
        }
    }
}
const ClientAtlasSqlOdbc = ClientAtlasSqlOdbcImpl;
exports.default = ClientAtlasSqlOdbc;
//# sourceMappingURL=index.js.map